package com.tocea.easycoverage.framework.checkers;

import java.util.ArrayList;
import java.util.List;

import com.tocea.easycoverage.framework.ClassChecker;
import com.tocea.easycoverage.framework.api.IInstanceProvider;
import com.tocea.easycoverage.framework.exception.EasycoverageAssertionError;
import com.tocea.easycoverage.framework.utils.Utils;

/**
 * 
 * Checkers that verifies that for any class implementing Comparable,
 * o1.compareTo(o2) is always -o2.compareTo(o1)
 * 
 * @author Romain Delamare
 * 
 */
public class BijectiveCompareToChecker extends ClassChecker {

	public BijectiveCompareToChecker(final IInstanceProvider provider,
			final Class<?> clazz) {
		super(provider, clazz);
	}

	@SuppressWarnings("unchecked")
	public void check() {
		if (provider != null && clazz != null) {
			final Object o1 = provider.getInstance(clazz);
			final Object o2 = provider.getInstance(clazz);
			if (((Comparable<Object>) o1).compareTo(o2) != -((Comparable<Object>) o2)
					.compareTo(o1))
				throw new EasycoverageAssertionError(
						"Comparable error: o1.compareTo(o2) should equal -o2.compareTo(o1).");
		}
	}

	@Override
	public boolean shouldCheck() {
		final List<Class<?>> interfaces = new ArrayList<Class<?>>();
		Utils.getAllInterfaces(clazz, interfaces);
		return super.shouldCheck() && interfaces.contains(Comparable.class);
	}

}
