package com.tocea.easycoverage.framework.checkers;

import java.lang.reflect.Method;

import com.tocea.easycoverage.framework.ClassChecker;
import com.tocea.easycoverage.framework.api.IInstanceProvider;
import com.tocea.easycoverage.framework.exception.EasycoverageAssertionError;

/**
 * 
 * Checkers that verifies that, for any class implementing equals, o1.equals(o2)
 * returns the same result as o2.equals(o1)
 * 
 * @author Romain Delamare
 * 
 */
public class BijectiveEqualsChecker extends ClassChecker {

	public BijectiveEqualsChecker(final IInstanceProvider provider,
			final Class<?> clazz) {
		super(provider, clazz);
	}

	public void check() {
		if (provider != null) {
			final Object o = provider.getInstance(clazz);
			if (!o.equals(o))
				throw new EasycoverageAssertionError(
						"o.equals(o) should always be true.");
		}
	}

	@Override
	public boolean shouldCheck() {
		Method method = null;
		if (clazz != null)
			try {
				method = clazz.getDeclaredMethod("equals", Object.class);
			} catch (final SecurityException e) {
				// nothing to do
			} catch (final NoSuchMethodException e) {
				// nothing to do
			}
		return super.shouldCheck() && method != null;
	}

}
