package com.tocea.easycoverage.framework.checkers;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.MethodChecker;
import com.tocea.easycoverage.framework.api.IInstanceProvider;
import com.tocea.easycoverage.framework.exception.EasycoverageAssertionError;
import com.tocea.easycoverage.framework.utils.Utils;

/**
 * 
 * Checkers that checks a getter does not return null. A getter is a method with
 * no parameters, not returning void, and starting with "is" or "has" (for
 * boolean return type) or "get" (for other return types).
 * 
 * @author Romain Delamare
 * 
 */
public class GetterNotNullChecker extends MethodChecker {

	private static final Logger	LOGGER	= LoggerFactory
												.getLogger(GetterNotNullChecker.class);

	public GetterNotNullChecker(final IInstanceProvider provider,
			final Method method) {
		super(provider, method);
	}

	public void check() {
		if (provider == null || method == null)
			return;
		final Object o = provider.getInstance(method.getDeclaringClass());
		try {
			final Object result = method.invoke(o);
			if (result == null)
				throw new EasycoverageAssertionError(
						"A getter should not return null.");
		} catch (final IllegalArgumentException e) {
			LOGGER.error("Error during invocation of {}: illegal arguments.",
					method);
		} catch (final IllegalAccessException e) {
			LOGGER.error("Error during invocation of {}: illegal access.",
					method);
		} catch (final InvocationTargetException e) {
			throw new RuntimeException(e.getCause());
		}
	}

	@Override
	public boolean shouldCheck() {
		return super.shouldCheck() && Utils.isGetter(method);
	}

}
