package com.tocea.easycoverage.framework.checkers;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import com.tocea.easycoverage.annotations.synchronize.NotSynchronized;
import com.tocea.easycoverage.annotations.synchronize.Synchronized;
import com.tocea.easycoverage.annotations.synchronize.SynchronizedBlock;
import com.tocea.easycoverage.framework.ClassChecker;
import com.tocea.easycoverage.framework.api.IInstanceProvider;
import com.tocea.easycoverage.framework.exception.EasycoverageAssertionError;
import com.tocea.easycoverage.framework.utils.Utils;

public class SynchronizedChecker extends ClassChecker {

	public SynchronizedChecker(final IInstanceProvider provider,
			final Class<?> clazz) {
		super(provider, clazz);
	}

	public void check() {
		if (clazz != null)
			for (final Method method : clazz.getDeclaredMethods())
				if (!Modifier.isSynchronized(method.getModifiers())
						&& !method.isAnnotationPresent(NotSynchronized.class)
						&& !method.isAnnotationPresent(SynchronizedBlock.class))
					throw new EasycoverageAssertionError("The method "
							+ Utils.getQualifiedName(method)
							+ " should be synchronized.");
	}

	@Override
	public boolean shouldCheck() {
		return super.shouldCheck()
				&& clazz.isAnnotationPresent(Synchronized.class);
	}

}
